import { clear, get, set } from 'ennea-tree';
import { makeFloodQueue } from './make';
import { WIRE, UNDERPASS, LIGHT, GATE, COMPONENT } from '../constants';
import { floodWire } from './wire';
import { floodUnderpass } from './underpass';
import { getUnderpassNeighbouringNets } from '../query/getNeighbouringNets';
import { makeWire } from '../actions/drawWire';
import light from './light';
import gate from './gate';
import component from './component';
export default function floodClear({ enneaTree, buddyTree }, x, y) {
    const item = get(enneaTree, y, x);
    return {
        enneaTree: floodClearInternal(clear(enneaTree, item).tree, [item.data, item]),
        buddyTree
    };
}
function floodClearInternal(enneaTree, ...floodSources) {
    const queue = [...makeFloodQueue(true, floodSources)];
    for (const { area, context } of queue) {
        enneaTree = clearWire(enneaTree, area, context, queue);
    }
    return enneaTree;
}
function clearWire(oldTree, area, context, queue) {
    const { tree: newTree, cleared: [item] } = clear(oldTree, area);
    if (!item)
        return newTree;
    const pos = { top: area.top - item.top, left: area.left - item.left };
    switch (item.data.type) {
        case WIRE:
            floodWire(context, queue);
            return newTree;
        case UNDERPASS:
            floodUnderpass(context, queue);
            const { vertical, horizontal } = getUnderpassNeighbouringNets(oldTree, item.left, item.top);
            const net = context.pos.top === context.prev.top ? vertical[0] : horizontal[0];
            if (net === undefined)
                return newTree;
            return set(newTree, makeWire(net), item);
        case LIGHT:
            return set(newTree, light(item.data, pos, context, queue), item);
        case GATE:
            return set(newTree, gate(item.data, pos, context, queue), item);
        case COMPONENT:
            return set(newTree, component(item.data, pos, context, queue), item);
        default:
            return oldTree;
    }
}
//# sourceMappingURL=floodClear.js.map