import { update } from 'ennea-tree';
import { WIRE, GATE, UNDERPASS, BUTTON, COMPONENT, LIGHT } from '../constants';
import wire from './wire';
import gate from './gate';
import underpass from './underpass';
import button from './button';
import component from './component';
import light from './light';
import unchanged from './unchanged';
import { makeFloodQueue } from './make';
export default function floodFill({ enneaTree, buddyTree }, item, pos) {
    return {
        enneaTree: floodFillInternal(enneaTree, false, [item, pos]),
        buddyTree
    };
}
export function floodFillGround(enenaTree, floodSources) {
    return floodFillInternal(enenaTree, true, ...floodSources);
}
function floodFillInternal(enneaTree, isGround = false, ...floodSources) {
    const queue = [...makeFloodQueue(isGround, floodSources)];
    const updater = update(enneaTree, (old, ctx, pos) => {
        switch (old.type) {
            case WIRE:
                return wire(old, pos, ctx, queue);
            case GATE:
                return gate(old, pos, ctx, queue);
            case UNDERPASS:
                return underpass(old, pos, ctx, queue);
            case BUTTON:
                return button(old, pos, ctx, queue);
            case COMPONENT:
                return component(old, pos, ctx, queue);
            case LIGHT:
                return light(old, pos, ctx, queue);
            default:
                return old;
        }
    }, unchanged);
    return updater.result(queue);
}
//# sourceMappingURL=floodFill.js.map