export default function upgradeComponent(component, size) {
    return from2to3(from1to2(from0to1(component)), size.width, size.height);
}
function from0to1(component) {
    if (component.schema === undefined) {
        return {
            type: 'component',
            schema: 1,
            inputs: component.inputs,
            outputs: component.outputs,
            gates: component.gates,
            nets: component.nets,
            name: component.name || 'unknown',
            repo: '',
            version: '0',
            hash: '0000000000000000000000000000000000000000'
        };
    }
    else {
        return component;
    }
}
function from1to2(component) {
    if (component.schema === 1) {
        return {
            type: 'component',
            schema: 2,
            inputs: component.inputs,
            outputs: component.outputs,
            gates: component.gates.map(({ inputA, inputB }) => [inputA, inputB]),
            net: component.nets[0],
            name: component.name,
            repo: component.repo,
            version: component.version,
            hash: component.hash
        };
    }
    else {
        return component;
    }
}
function from2to3(component, width, height) {
    if (component.schema === 2) {
        return {
            type: 'component',
            schema: 3,
            net: component.net,
            inputs: component.inputs.map(input => ({
                name: input.name,
                net: input.net,
                input: 0
            })),
            outputs: component.outputs.map(output => ({
                name: output.name,
                net: output.net
            })),
            package: {
                hash: component.hash,
                name: component.name,
                repo: component.repo,
                version: component.version,
                height,
                width,
                inputs: component.inputs.map((input, index) => ({
                    x: input.x,
                    y: input.y,
                    dx: input.dx,
                    dy: input.dy,
                    group: index
                })),
                outputs: component.outputs.map(output => ({
                    x: output.x,
                    y: output.y,
                    dx: output.dx,
                    dy: output.dy,
                    name: output.name || '',
                    gate: output.net - component.net
                })),
                gates: component.gates.map(gate => [
                    gate[0] < component.net ? 'GROUND' : gate[0] - component.net,
                    gate[1] < component.net ? 'GROUND' : gate[1] - component.net
                ]),
                groups: component.inputs.map(input => ({
                    name: input.name || '',
                    pointsTo: input.pointsTo.map(point => ({
                        gate: point.index,
                        input: point.input
                    }))
                }))
            }
        };
    }
    else {
        return component;
    }
}
//# sourceMappingURL=component.js.map