"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = drawComponent;
exports.getNetAtPos = getNetAtPos;

var _newArrowCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/newArrowCheck"));

var buddy = _interopRequireWildcard(require("buddy-tree"));

var _constants = require("../constants");

var _getNetAt = _interopRequireDefault(require("../query/getNetAt"));

var _insertItem = _interopRequireDefault(require("./insertItem"));

var _utils = require("../utils");

var COMPONENT_SCHEMA = 3;

function drawComponent(forest, x, y, pkg) {
  var _this = this;

  x -= pkg.width >> 1;
  y -= pkg.height >> 1;

  var _buddy$allocate = buddy.allocate(forest.buddyTree, pkg.gates.length),
      buddyTree = _buddy$allocate.tree,
      address = _buddy$allocate.address;

  var groups = pkg.groups.map(function (group, index) {
    var _this2 = this;

    (0, _newArrowCheck2["default"])(this, _this);
    return {
      name: group.name,
      index: index,
      nets: pkg.inputs.filter(function (input) {
        (0, _newArrowCheck2["default"])(this, _this2);
        return input.group === index;
      }.bind(this)).map(function (input, index) {
        (0, _newArrowCheck2["default"])(this, _this2);
        return {
          input: index,
          net: getNetAtPos(forest.enneaTree, x, y, input.x, input.y, input.dx, input.dy)
        };
      }.bind(this)).filter(function (pin) {
        (0, _newArrowCheck2["default"])(this, _this2);
        return pin.net !== _constants.GROUND;
      }.bind(this))
    };
  }.bind(this));

  if (groups.some(function (group) {
    var _this3 = this;

    (0, _newArrowCheck2["default"])(this, _this);
    return group.nets.map(function (g) {
      (0, _newArrowCheck2["default"])(this, _this3);
      return g.net;
    }.bind(this)).filter(_utils.distinct).length > 1;
  }.bind(this))) {
    return forest;
  }

  var inputs = groups.map(function (group) {
    (0, _newArrowCheck2["default"])(this, _this);
    return {
      net: group.nets.length === 0 ? _constants.GROUND : group.nets[0].net,
      name: group.name,
      input: group.nets.length === 0 ? -1 : group.nets[0].input
    };
  }.bind(this));
  var outputs = pkg.outputs.map(function (output) {
    (0, _newArrowCheck2["default"])(this, _this);
    return {
      net: address + output.gate,
      name: output.name
    };
  }.bind(this));
  var data = {
    type: _constants.COMPONENT,
    schema: COMPONENT_SCHEMA,
    net: address,
    inputs: inputs,
    outputs: outputs,
    "package": pkg
  };
  var box = {
    left: x,
    top: y,
    width: pkg.width,
    height: pkg.height
  };
  return (0, _insertItem["default"])(forest, buddyTree, data, box);
}

function getNetAtPos(tree, sx, sy, x, y, dx, dy) {
  return (0, _getNetAt["default"])(tree, sx + x + dx, sy + y + dy, dx, dy);
}
//# sourceMappingURL=drawComponent.js.map