"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = floodClear;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _enneaTree = require("ennea-tree");

var _make = require("./make");

var _constants = require("../constants");

var _wire = require("./wire");

var _underpass = require("./underpass");

var _getNeighbouringNets = require("../query/getNeighbouringNets");

var _drawWire = require("../actions/drawWire");

var _light = _interopRequireDefault(require("./light"));

var _gate = _interopRequireDefault(require("./gate"));

var _component = _interopRequireDefault(require("./component"));

function floodClear(_ref, x, y) {
  var enneaTree = _ref.enneaTree,
      buddyTree = _ref.buddyTree;
  var item = (0, _enneaTree.get)(enneaTree, y, x);
  return {
    enneaTree: floodClearInternal((0, _enneaTree.clear)(enneaTree, item).tree, [item.data, item]),
    buddyTree: buddyTree
  };
}

function floodClearInternal(enneaTree) {
  for (var _len = arguments.length, floodSources = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    floodSources[_key - 1] = arguments[_key];
  }

  var queue = (0, _toConsumableArray2["default"])((0, _make.makeFloodQueue)(true, floodSources));
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = queue[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ref3 = _step.value;
      var area = _ref3.area,
          context = _ref3.context;
      enneaTree = clearWire(enneaTree, area, context, queue);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return enneaTree;
}

function clearWire(oldTree, area, context, queue) {
  var _clear = (0, _enneaTree.clear)(oldTree, area),
      newTree = _clear.tree,
      _clear$cleared = (0, _slicedToArray2["default"])(_clear.cleared, 1),
      item = _clear$cleared[0];

  if (!item) return newTree;
  var pos = {
    top: area.top - item.top,
    left: area.left - item.left
  };

  switch (item.data.type) {
    case _constants.WIRE:
      (0, _wire.floodWire)(context, queue);
      return newTree;

    case _constants.UNDERPASS:
      (0, _underpass.floodUnderpass)(context, queue);

      var _getUnderpassNeighbou = (0, _getNeighbouringNets.getUnderpassNeighbouringNets)(oldTree, item.left, item.top),
          vertical = _getUnderpassNeighbou.vertical,
          horizontal = _getUnderpassNeighbou.horizontal;

      var net = context.pos.top === context.prev.top ? vertical[0] : horizontal[0];
      if (net === undefined) return newTree;
      return (0, _enneaTree.set)(newTree, (0, _drawWire.makeWire)(net), item);

    case _constants.LIGHT:
      return (0, _enneaTree.set)(newTree, (0, _light["default"])(item.data, pos, context, queue), item);

    case _constants.GATE:
      return (0, _enneaTree.set)(newTree, (0, _gate["default"])(item.data, pos, context, queue), item);

    case _constants.COMPONENT:
      return (0, _enneaTree.set)(newTree, (0, _component["default"])(item.data, pos, context, queue), item);

    default:
      return oldTree;
  }
}
//# sourceMappingURL=floodClear.js.map