"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = compile;
exports.ensureNetExists = ensureNetExists;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _newArrowCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/newArrowCheck"));

var ennea = _interopRequireWildcard(require("ennea-tree"));

var _constants = require("../constants");

var _utils = require("../utils");

var _layoutPins = _interopRequireDefault(require("./layoutPins"));

function compile(forest, repo, name, version, hash) {
  var _this = this;

  var enneaTree = forest.enneaTree;
  var forestContet = ennea.getAll(enneaTree, {
    top: 0,
    left: 0,
    width: enneaTree.size,
    height: enneaTree.size
  });
  var forestButtons = forestContet.filter(function (node) {
    (0, _newArrowCheck2["default"])(this, _this);
    return node.data.type === _constants.BUTTON;
  }.bind(this));
  var forestGates = forestContet.filter(function (node) {
    (0, _newArrowCheck2["default"])(this, _this);
    return node.data.type === _constants.GATE;
  }.bind(this)).map(function (node) {
    (0, _newArrowCheck2["default"])(this, _this);
    return node.data;
  }.bind(this));
  var forestComponents = forestContet.filter(function (node) {
    (0, _newArrowCheck2["default"])(this, _this);
    return node.data.type === _constants.COMPONENT;
  }.bind(this)).map(function (node) {
    (0, _newArrowCheck2["default"])(this, _this);
    return node.data;
  }.bind(this));
  var forestLights = forestContet.filter(function (node) {
    (0, _newArrowCheck2["default"])(this, _this);
    return node.data.type === _constants.LIGHT && node.data.net !== _constants.GROUND;
  }.bind(this));
  return Object.assign({}, createPackage(forestGates, forestComponents, forestButtons, forestLights), {
    name: name,
    hash: hash,
    repo: repo,
    version: version
  });
}

function createPackage(forestGates, forestComponents, forestButtons, forestLights) {
  var _this2 = this;

  var gateAddresses = [].concat((0, _toConsumableArray2["default"])(forestGates.map(makeGate)), (0, _toConsumableArray2["default"])(forestComponents.map(makeGatesFromComponent).reduce(_utils.flatten, [])));
  var netToGateIndex = new Map(gateAddresses.map(function (gate, index) {
    (0, _newArrowCheck2["default"])(this, _this2);
    return [gate.net, index];
  }.bind(this)));
  var netToButtonIndex = new Map(forestButtons.map(function (button, index) {
    (0, _newArrowCheck2["default"])(this, _this2);
    return [button.data.net, index];
  }.bind(this)));
  var layout = (0, _layoutPins["default"])(forestButtons.map(toPin).concat(forestLights.map(toPin).map(reverseDirection).filter(function (pin) {
    (0, _newArrowCheck2["default"])(this, _this2);
    return netToButtonIndex.has(pin.net);
  }.bind(this))), forestLights.map(toPin).filter(function (pin) {
    (0, _newArrowCheck2["default"])(this, _this2);
    return netToGateIndex.has(pin.net);
  }.bind(this)));
  var inputs = layout.inputs.map(function (pin) {
    (0, _newArrowCheck2["default"])(this, _this2);
    return {
      group: netToButtonIndex.get(pin.net) || 0,
      x: pin.x,
      y: pin.y,
      dx: -pin.dx,
      dy: -pin.dy
    };
  }.bind(this));
  var gates = gateAddresses.map(function (gate) {
    (0, _newArrowCheck2["default"])(this, _this2);
    return [groundIfUndefined(netToGateIndex.get(gate.a)), groundIfUndefined(netToGateIndex.get(gate.b))];
  }.bind(this));
  var outputs = layout.outputs.map(function (pin) {
    (0, _newArrowCheck2["default"])(this, _this2);
    return {
      gate: netToGateIndex.get(pin.net),
      x: pin.x,
      y: pin.y,
      dx: pin.dx,
      dy: pin.dy,
      name: pin.name
    };
  }.bind(this)).filter(function (node) {
    (0, _newArrowCheck2["default"])(this, _this2);
    return node.gate !== undefined;
  }.bind(this));
  var groups = forestButtons.map(function (button) {
    var _this3 = this;

    (0, _newArrowCheck2["default"])(this, _this2);
    return {
      name: button.data.name,
      pointsTo: [].concat((0, _toConsumableArray2["default"])(gateAddresses.filter(function (g) {
        (0, _newArrowCheck2["default"])(this, _this3);
        return g.a === button.data.net;
      }.bind(this)).map(makeInput('A', netToGateIndex))), (0, _toConsumableArray2["default"])(gateAddresses.filter(function (g) {
        (0, _newArrowCheck2["default"])(this, _this3);
        return g.b === button.data.net;
      }.bind(this)).map(makeInput('B', netToGateIndex))))
    };
  }.bind(this));
  return {
    width: layout.width,
    height: layout.height,
    inputs: inputs,
    outputs: outputs,
    gates: gates,
    groups: groups
  };
}

function makeGate(_ref) {
  var net = _ref.net,
      a = _ref.inputA,
      b = _ref.inputB;
  return {
    net: net,
    a: a,
    b: b
  };
}

function makeGatesFromComponent(component) {
  var _this4 = this;

  var gates = component["package"].gates;
  var inputs = component.inputs;
  var pins = component["package"].groups.map(function (input, index) {
    var _this5 = this;

    (0, _newArrowCheck2["default"])(this, _this4);
    return input.pointsTo.map(function (_ref2) {
      var gate = _ref2.gate,
          input = _ref2.input;
      (0, _newArrowCheck2["default"])(this, _this5);
      return {
        gate: gate,
        net: inputs[index].net,
        input: input
      };
    }.bind(this));
  }.bind(this)).reduce(_utils.flatten, []);
  var pointsToA = new Map(pins.filter(function (p) {
    (0, _newArrowCheck2["default"])(this, _this4);
    return p.input === 'A';
  }.bind(this)).map(function (p) {
    (0, _newArrowCheck2["default"])(this, _this4);
    return [p.gate, p.net];
  }.bind(this)));
  var pointsToB = new Map(pins.filter(function (p) {
    (0, _newArrowCheck2["default"])(this, _this4);
    return p.input === 'B';
  }.bind(this)).map(function (p) {
    (0, _newArrowCheck2["default"])(this, _this4);
    return [p.gate, p.net];
  }.bind(this)));
  return gates.map(function (_ref3, index) {
    var _ref4 = (0, _slicedToArray2["default"])(_ref3, 2),
        a = _ref4[0],
        b = _ref4[1];

    (0, _newArrowCheck2["default"])(this, _this4);
    return {
      net: component.net + index,
      a: a !== 'GROUND' ? component.net + a : pointsToA.get(index) || _constants.GROUND,
      b: b !== 'GROUND' ? component.net + b : pointsToB.get(index) || _constants.GROUND
    };
  }.bind(this));
}

function ensureNetExists(input) {
  if (input == undefined) {
    throw new Error("could not find any net");
  }

  return input;
}

function makeInput(input, netToIndexMap) {
  var _this6 = this;

  return function (gate) {
    (0, _newArrowCheck2["default"])(this, _this6);
    return {
      input: input,
      gate: netToIndexMap.get(gate.net) || 0
    };
  }.bind(this);
}

function toPin(_ref5) {
  var y = _ref5.top,
      x = _ref5.left,
      _ref5$data = _ref5.data,
      direction = _ref5$data.direction,
      net = _ref5$data.net,
      name = _ref5$data.name;
  return {
    x: x,
    y: y,
    dx: (0, _utils.directionToDx)(direction),
    dy: (0, _utils.directionToDy)(direction),
    net: net,
    name: name
  };
}

;

function reverseDirection(pin) {
  return Object.assign({}, pin, {
    dx: -pin.dx,
    dy: -pin.dy
  });
}

function groundIfUndefined(a) {
  return a === undefined ? 'GROUND' : a;
}
//# sourceMappingURL=index.js.map