"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = getNetAt;
exports.getGateNet = getGateNet;
exports.getUnderpassNet = getUnderpassNet;
exports.getButtonNet = getButtonNet;
exports.getComponentNet = getComponentNet;

var _newArrowCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/newArrowCheck"));

var _enneaTree = require("ennea-tree");

var _constants = require("../constants");

function getNetAt(enneaTree, x, y, dx, dy) {
  var tile = (0, _enneaTree.get)(enneaTree, y, x);
  if (!tile || !tile.data) return _constants.GROUND;

  switch (tile.data.type) {
    case _constants.WIRE:
      return tile.data.net;

    case _constants.GATE:
      return getGateNet(tile.data, x - tile.left, y - tile.top, dx, dy);

    case _constants.UNDERPASS:
      return getUnderpassNet(tile.data, dx, dy, enneaTree, x, y);

    case _constants.BUTTON:
      return getButtonNet(tile.data, x - tile.left, y - tile.top, dx, dy);

    case _constants.COMPONENT:
      return getComponentNet(tile.data, x - tile.left, y - tile.top, dx, dy);

    default:
      return _constants.GROUND;
  }
}

function getGateNet(gate, x, y, dx, dy) {
  if (x === 3 && y === 1 && dx === -1 && dy === 0) {
    return gate.net;
  } else {
    return _constants.GROUND;
  }
}

function getUnderpassNet(underpass, dx, dy, enneaTree, x, y) {
  if (dx !== 0 && dy === 0) {
    return underpass.net;
  } else {
    return getNetAt(enneaTree, x, y + dy, dx, dy);
  }
}

function getButtonNet(button, x, y, dx, dy) {
  switch (button.direction) {
    case 'upwards':
      return x === 1 && y === 0 && dx === 0 && dy === 1 ? button.net : _constants.GROUND;

    case 'downwards':
      return x === 1 && y === 2 && dx === 0 && dy === -1 ? button.net : _constants.GROUND;

    case 'leftwards':
      return x === 0 && y === 1 && dx === 1 && dy === 0 ? button.net : _constants.GROUND;

    case 'rightwards':
    default:
      return x === 2 && y === 1 && dx === -1 && dy === 0 ? button.net : _constants.GROUND;
  }
}

function getComponentNet(component, x, y, dx, dy) {
  var _this = this;

  var index = component["package"].outputs.findIndex(function (output) {
    (0, _newArrowCheck2["default"])(this, _this);
    return output.x === x && output.y === y && output.dx === -dx && output.dy === -dy;
  }.bind(this));

  if (index >= 0) {
    return component.outputs[index].net;
  }

  var input = component["package"].inputs.find(function (input) {
    (0, _newArrowCheck2["default"])(this, _this);
    return input.x === x && input.y === y && input.dx === -dx && input.dy === -dy;
  }.bind(this));

  if (input) {
    return component.inputs[input.group].net;
  }

  return _constants.GROUND;
}
//# sourceMappingURL=getNetAt.js.map