"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = set;
exports.wire = wire;
exports.gate = gate;
exports.underpass = underpass;
exports.button = button;
exports.component = component;
exports.light = light;

var _newArrowCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/newArrowCheck"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _constants = require("../constants");

var _utils = require("../utils");

var tile = _interopRequireWildcard(require("./tile"));

function set(context, change) {
  switch (change.after.type) {
    case _constants.WIRE:
      return wire(context, change, change.after);

    case _constants.GATE:
      return gate(context, change, change.after);

    case _constants.UNDERPASS:
      return underpass(context, change, change.after);

    case _constants.BUTTON:
      return button(context, change, change.after);

    case _constants.COMPONENT:
      return component(context, change, change.after);

    case _constants.LIGHT:
      return light(context, change, change.after);
  }
}

function wire(context, _ref, wire) {
  var y = _ref.top,
      x = _ref.left;
  context.setMap(x, y, tile.wire());
  context.setNet(x, y, wire.net);
}

function gate(context, _ref2, gate) {
  var y = _ref2.top,
      x = _ref2.left,
      width = _ref2.width,
      height = _ref2.height;

  for (var ty = 0; ty < height; ty++) {
    for (var tx = 0; tx < width; tx++) {
      context.setMap(tx + x, ty + y, tile.gate(tx, ty));
    }
  }

  context.setNet(x + 3, y + 1, gate.net);
  context.setNet(x + 0, y + 0, gate.inputA);
  context.setNet(x + 0, y + 2, gate.inputB);
  context.setGate(gate.net, gate.inputA, gate.inputB);
}

function underpass(context, _ref3, underpass) {
  var y = _ref3.top,
      x = _ref3.left;
  context.setMap(x, y, tile.underpass());
  context.setNet(x, y, underpass.net);
}

function button(context, _ref4, button) {
  var y = _ref4.top,
      x = _ref4.left,
      width = _ref4.width,
      height = _ref4.height;
  var dx = (0, _utils.directionToDx)(button.direction);
  var dy = (0, _utils.directionToDy)(button.direction);

  for (var ty = 0; ty < height; ty++) {
    for (var tx = 0; tx < width; tx++) {
      if (tx - 1 === dx && ty - 1 === dy) {
        context.setMap(tx + x, ty + y, tile.buttonOutput(dx, dy));
      } else {
        context.setMap(tx + x, ty + y, tile.button(tx, ty));
      }

      context.setNet(tx + x, ty + y, button.net);
    }
  }

  context.setGate(button.net, 1, 1);
}

function component(context, _ref5, component) {
  var _this = this;

  var y = _ref5.top,
      x = _ref5.left,
      width = _ref5.width,
      height = _ref5.height;
  var pins = [].concat((0, _toConsumableArray2["default"])(component["package"].inputs), (0, _toConsumableArray2["default"])(component["package"].outputs));

  for (var ty = 0; ty < height; ty++) {
    for (var tx = 0; tx < width; tx++) {
      context.setMap(tx + x, ty + y, tile.component(tx, ty, width - 1, height - 1, pins));
    }
  }

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _utils.zip)(component["package"].outputs, component.outputs)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _step$value = (0, _slicedToArray2["default"])(_step.value, 2),
          pin = _step$value[0],
          port = _step$value[1];

      context.setNet(x + pin.x, y + pin.y, port.net);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = component["package"].inputs[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var pin = _step2.value;
      var port = component.inputs[pin.group];
      context.setNet(x + pin.x, y + pin.y, port.net);
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
        _iterator2["return"]();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  var _iteratorNormalCompletion3 = true;
  var _didIteratorError3 = false;
  var _iteratorError3 = undefined;

  try {
    for (var _iterator3 = component["package"].gates.map(function (_ref8, index) {
      var _ref9 = (0, _slicedToArray2["default"])(_ref8, 2),
          a = _ref9[0],
          b = _ref9[1];

      (0, _newArrowCheck2["default"])(this, _this);
      return {
        a: a,
        b: b,
        index: index
      };
    }.bind(this))[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
      var _ref7 = _step3.value;
      var a = _ref7.a,
          b = _ref7.b,
          index = _ref7.index;
      context.setGate(component.net + index, component.net + (a === 'GROUND' ? 0 : a), component.net + (b === 'GROUND' ? 0 : b));
    }
  } catch (err) {
    _didIteratorError3 = true;
    _iteratorError3 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion3 && _iterator3["return"] != null) {
        _iterator3["return"]();
      }
    } finally {
      if (_didIteratorError3) {
        throw _iteratorError3;
      }
    }
  }

  var _iteratorNormalCompletion4 = true;
  var _didIteratorError4 = false;
  var _iteratorError4 = undefined;

  try {
    for (var _iterator4 = (0, _utils.zip)(component["package"].groups, component.inputs)[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
      var _step4$value = (0, _slicedToArray2["default"])(_step4.value, 2),
          group = _step4$value[0],
          _port = _step4$value[1];

      var _iteratorNormalCompletion5 = true;
      var _didIteratorError5 = false;
      var _iteratorError5 = undefined;

      try {
        for (var _iterator5 = group.pointsTo[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
          var point = _step5.value;

          if (point.input === 'A') {
            context.setGateA(component.net + point.gate, _port.net);
          } else {
            context.setGateB(component.net + point.gate, _port.net);
          }
        }
      } catch (err) {
        _didIteratorError5 = true;
        _iteratorError5 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion5 && _iterator5["return"] != null) {
            _iterator5["return"]();
          }
        } finally {
          if (_didIteratorError5) {
            throw _iteratorError5;
          }
        }
      }
    }
  } catch (err) {
    _didIteratorError4 = true;
    _iteratorError4 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion4 && _iterator4["return"] != null) {
        _iterator4["return"]();
      }
    } finally {
      if (_didIteratorError4) {
        throw _iteratorError4;
      }
    }
  }

  context.insertText(component, {
    top: y,
    left: x,
    width: width,
    height: height
  });
}

function light(context, _ref10, light) {
  var y = _ref10.top,
      x = _ref10.left,
      width = _ref10.width,
      height = _ref10.height;
  var dx = (0, _utils.directionToDx)(light.direction);
  var dy = (0, _utils.directionToDy)(light.direction);

  for (var ty = 0; ty < height; ty++) {
    for (var tx = 0; tx < width; tx++) {
      if (dy === 0 && tx - 1 === -dx && ty === 1) {
        context.setMap(tx + x, ty + y, tile.lightInput(dx, dy));
      } else if (dx === 0 && tx === 1 && ty - 1 === -dy) {
        context.setMap(tx + x, ty + y, tile.lightInput(dx, dy));
      } else {
        context.setMap(tx + x, ty + y, tile.light(tx, ty));
      }

      context.setNet(tx + x, ty + y, light.net);
    }
  }
}
//# sourceMappingURL=set.js.map