"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = update;
exports.updateWire = updateWire;
exports.updateGate = updateGate;
exports.updateUnderpass = updateUnderpass;
exports.updateButton = updateButton;
exports.updateLight = updateLight;
exports.updateComponent = updateComponent;

var _newArrowCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/newArrowCheck"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _enneaTree = require("ennea-tree");

var _set = _interopRequireDefault(require("./set"));

var _clear = _interopRequireDefault(require("./clear"));

var _constants = require("../constants");

var _utils = require("../utils");

var tile = _interopRequireWildcard(require("./tile"));

function update(context, change) {
  switch (change.after.type) {
    case _constants.WIRE:
      return updateWire(context, change, change.after);

    case _constants.GATE:
      return updateGate(context, change, change.before, change.after);

    case _constants.UNDERPASS:
      return updateUnderpass(context, change, change.after);

    case _constants.BUTTON:
      return updateButton(context, change, change.before, change.after);

    case _constants.LIGHT:
      return updateLight(context, change, change.before, change.after);

    case _constants.COMPONENT:
      if (change.before["package"].hash === change.after["package"].hash) {
        return updateComponent(context, change, change.before, change.after);
      } else {
        var top = change.top,
            left = change.left,
            width = change.width,
            height = change.height;
        (0, _clear["default"])(context, {
          before: change.before,
          top: top,
          left: left,
          width: width,
          height: height,
          type: _enneaTree.CLEAR
        });
        (0, _set["default"])(context, {
          after: change.after,
          top: top,
          left: left,
          width: width,
          height: height,
          type: _enneaTree.SET
        });
      }

  }
}

function updateWire(context, _ref, wire) {
  var y = _ref.top,
      x = _ref.left;
  context.setMap(x, y, tile.wire());
  context.setNet(x, y, wire.net);
}

function updateGate(context, _ref2, oldGate, newGate) {
  var y = _ref2.top,
      x = _ref2.left;
  var changed = false;

  if (oldGate.net !== newGate.net) {
    context.setNet(x + 3, y + 1, newGate.net);
    changed = true;
  }

  if (oldGate.inputA !== newGate.inputA) {
    context.setNet(x, y + 0, newGate.inputA);
    changed = true;
  }

  if (oldGate.inputB !== newGate.inputB) {
    context.setNet(x, y + 2, newGate.inputB);
    changed = true;
  }

  if (changed) {
    context.setGate(newGate.net, newGate.inputA, newGate.inputB);
  }
}

function updateUnderpass(context, _ref3, underpass) {
  var y = _ref3.top,
      x = _ref3.left;
  context.setMap(x, y, tile.underpass());
  context.setNet(x, y, underpass.net);
}

function updateButton(context, _ref4, oldButton, newButton) {
  var y = _ref4.top,
      x = _ref4.left,
      width = _ref4.width,
      height = _ref4.height;

  if (oldButton.direction !== newButton.direction) {
    var oldDx = (0, _utils.directionToDx)(oldButton.direction);
    var oldDy = (0, _utils.directionToDy)(oldButton.direction);
    var newDx = (0, _utils.directionToDx)(newButton.direction);
    var newDy = (0, _utils.directionToDy)(newButton.direction);
    context.setMap(x + oldDx + 1, y + oldDy + 1, tile.button(oldDx + 1, oldDy + 1));
    context.setMap(x + newDx + 1, y + newDy + 1, tile.buttonOutput(newDx, newDy));
  }

  if (oldButton.net !== newButton.net) {
    for (var ty = 0; ty < height; ty++) {
      for (var tx = 0; tx < width; tx++) {
        context.setNet(tx + x, ty + y, newButton.net);
      }
    }
  }

  context.setGate(newButton.net, 1, 1);
}

function updateLight(context, _ref5, oldLight, newLight) {
  var y = _ref5.top,
      x = _ref5.left,
      width = _ref5.width,
      height = _ref5.height;

  if (oldLight.direction !== newLight.direction) {
    var oldDx = (0, _utils.directionToDx)(oldLight.direction);
    var oldDy = (0, _utils.directionToDy)(oldLight.direction);
    var newDx = (0, _utils.directionToDx)(newLight.direction);
    var newDy = (0, _utils.directionToDy)(newLight.direction);
    context.setMap(x - oldDx + 1, y - oldDy + 1, tile.light(-oldDx + 1, -oldDy + 1));
    context.setMap(x - newDx + 1, y - newDy + 1, tile.lightInput(newDx, newDy));
  }

  if (oldLight.net !== newLight.net) {
    for (var ty = 0; ty < height; ty++) {
      for (var tx = 0; tx < width; tx++) {
        context.setNet(tx + x, ty + y, newLight.net);
      }
    }
  }
}

function updateComponent(context, _ref6, oldComponent, newComponent) {
  var y = _ref6.top,
      x = _ref6.left;
  var oldInputs = oldComponent.inputs;
  var newInputs = newComponent.inputs;
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    var _loop = function _loop() {
      var _this = this;

      var _step$value = (0, _slicedToArray2["default"])(_step.value, 3),
          oldInput = _step$value[0],
          newInput = _step$value[1],
          index = _step$value[2];

      if (oldInput.net !== newInput.net) {
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = oldComponent["package"].inputs.filter(function (input) {
            (0, _newArrowCheck2["default"])(this, _this);
            return input.group === index;
          }.bind(this))[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var pin = _step2.value;
            context.setNet(x + pin.x, y + pin.y, newInput.net);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
              _iterator2["return"]();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          for (var _iterator3 = oldComponent["package"].groups[index].pointsTo[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var point = _step3.value;

            if (point.input === 'A') {
              context.setGateA(oldComponent.net + point.gate, newInput.net);
            } else {
              context.setGateB(oldComponent.net + point.gate, newInput.net);
            }
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3["return"] != null) {
              _iterator3["return"]();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }
      }
    };

    for (var _iterator = (0, _utils.zip)(oldInputs, newInputs)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      _loop();
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  context.updateText(oldComponent, newComponent);
}
//# sourceMappingURL=update.js.map